#include <stdio.h>
#include <math.h>

#include "../include/complex.h"

int InitCmplx(Cmplx *ptr, double real, double imaginary){
    ptr->real = real;
    ptr->imag = imaginary;
    return 0;
}

int DisplayCmplx(Cmplx *complex) {
    if(complex->imag >= 0) {
        printf("%lf + %lfj\n", complex->real, complex->imag);
    } else {
        printf("%lf - %lfj\n", complex->real, -(complex->imag));
    }
    return 0;
}

int GetCmplx(Cmplx *ptr) {

    double real;
    double imaginary;

    printf("Entrez la partie réelle : ");
    scanf("%lf", &real);

    printf("Entrez la partie imaginaire : ");
    scanf("%lf", &imaginary);

    InitCmplx(ptr, real, imaginary);

    return 0;
}

int ConjCmplx(Cmplx *ptr, Cmplx *conjugate) {
    InitCmplx(conjugate, ptr->real, -(ptr->imag));

    return 0;
}

int ModCmplx(Cmplx *ptrCmplx, double *ptrDouble) {
    *ptrDouble = sqrt(ptrCmplx->real*ptrCmplx->real + ptrCmplx->imag*ptrCmplx->imag);
    return 0;
}

int ArgCmplx(Cmplx *ptrCmplx, double *ptrDouble) {
    if(ptrCmplx->real > 0) {
        *ptrDouble = atan2(ptrCmplx->imag, ptrCmplx->real);
    } else if(ptrCmplx->real == 0 && ptrCmplx->imag > 0) {
        *ptrDouble = 3.14159/2;
    } else if(ptrCmplx->real == 0 && ptrCmplx->imag < 0) {
        *ptrDouble = -3.14159/2;
    } else if(ptrCmplx->imag == 0 && ptrCmplx->real == 0) {
        *ptrDouble = 0; 
    } else {
        *ptrDouble = 3.14159 + atan2(ptrCmplx->imag, ptrCmplx->real);
    }
}

int SumCmplx(Cmplx *ptr1, Cmplx *ptr2, Cmplx *newPtr) {
    InitCmplx(newPtr, ptr1->real + ptr2->real, ptr1->imag + ptr2->imag);
    return 0; 
}

int SubCmplx(Cmplx *ptr1, Cmplx *ptr2, Cmplx *newPtr) {
    InitCmplx(newPtr, ptr1->real - ptr2->real, ptr1->imag - ptr2->imag);
    return 0; 
}

int MulCmplx(Cmplx *ptr1, Cmplx *ptr2, Cmplx *newPtr) {
    double newReal = (ptr1->real)*(ptr2->real) - (ptr1->imag)*(ptr2->imag);
    double newImaginary = (ptr1->imag)*(ptr2->real) + (ptr1->real)*(ptr2->imag); 

    InitCmplx(newPtr, newReal, newImaginary);
    return 0; 
}

int ScalarMulCmplx(Cmplx *ptr, double scalar, Cmplx *newPtr) {
    InitCmplx(newPtr, ptr->real*scalar, ptr->imag*scalar);
}

int DivCmplx(Cmplx *ptr1, Cmplx *ptr2, Cmplx *newPtr) {
    // Divise ptr1 par ptr2
    double tempMod;
    ModCmplx(ptr2, &tempMod);
    tempMod = tempMod * tempMod;

    Cmplx tempConjugate;
    ConjCmplx(ptr2, &tempConjugate);
    MulCmplx(ptr1, &tempConjugate, newPtr);

    newPtr->real = newPtr->real / tempMod ;
    newPtr->imag = newPtr->imag / tempMod ;
    return 0; 
}

int PowCmplx(Cmplx *ptr, int power, Cmplx *newPtr) {
    if(power == 0) {
        InitCmplx(newPtr, 1, 0);
        return 0;
    } else if(power < 0) {
        printf("Mets un entier positif stp\n");
        return 0;
    } else {
        MulCmplx(ptr, ptr, newPtr);
        for(int i = 2; i < power; i++) {
            MulCmplx(newPtr, ptr, newPtr);
        }
        return 0;
    }
}

int EvalPoly(double *coeff, int degree, Cmplx *point, Cmplx *result) {
    InitCmplx(result, coeff[0] + coeff[1]*point->real, coeff[1]*point->imag);

    for (int i = 2; i < degree; i++) {
        Cmplx tempCmplx;
        // Met le point à la puissance correspondante
        PowCmplx(point, i, &tempCmplx);
        // Multiplie par le coefficient
        ScalarMulCmplx(&tempCmplx, coeff[i], &tempCmplx);
        // Ajoute le produit au résultat
        SumCmplx(result, &tempCmplx, result);
    }
    return 0;
}
